@@ -913,40 +913,121 @@ class Connector:
 
     @property
     def get_grid(self) -> List[List[CssColorType]]:
+        '''
+        returns a copy of the last sent grid. Changes on the returned grid will not have an effect.
+        '''
+        return deepcopy(self.__get_grid)
+
+    @property
+    def __get_grid(self) -> List[List[CssColorType]]:
+        '''
+        returns the internal grid (no deep copy!)
+        '''
         grid = self.__last_sent_grid['grid']
         is_2d = len(grid) > 0 and type(grid[0]) != str and hasattr(grid[0], "__getitem__")
         if is_2d:
-            return deepcopy(grid)
-        return [deepcopy(grid)]
+            return grid
+        return [grid]
 
-    def update_cell(self, row: Optional[int] = None, column: Optional[int] = None, color: Optional[CssColorType] = None, cellNumber: Optional[int] = None, base_color: Optional[BaseColor] = None, enumerate: bool = None, **delivery_opts):
-        '''
-        sets the color of the current grid at the given row and column
-        '''
-        self.update_grid(row=row, column=column, color=color, cellNumber=cellNumber,
-                         base_color=base_color, enumerate=enumerate, **delivery_opts)
+    def get_grid_at(self, row: Optional[int] = None, column: Optional[int] = None, cell_number: Optional[int] = None) -> Optional[CssColorType]:
+        grid = self.__get_grid
+        if cell_number is not None:
+            row = (cell_number - 1) // len(grid[0])
+            column = (cell_number - 1) % len(grid[0])
+        elif row is None or column is None:
+            return None
 
-    def set_grid_at(self, row: Optional[int] = None, column: Optional[int] = None, color: Optional[CssColorType] = None, cellNumber: Optional[int] = None, base_color: Optional[BaseColor] = None, enumerate: bool = None, **delivery_opts):
-        '''
-        sets the color of the current grid at the given row and column
-        '''
-        self.update_grid(row=row, column=column, color=color, cellNumber=cellNumber,
-                         base_color=base_color, enumerate=enumerate, **delivery_opts)
+        if len(grid) <= row:
+            return None
+        if len(grid[row]) <= column:
+            return None
+        return grid[row][column]
 
-    def update_grid(self, row: Optional[int] = None, column: Optional[int] = None, color: Optional[CssColorType] = None, cellNumber: Optional[int] = None, base_color: Optional[BaseColor] = None, enumerate: bool = None, **delivery_opts):
+    def update_cell(self, row: Optional[int] = None, column: Optional[int] = None, color: Optional[CssColorType] = None, cell_number: Optional[int] = None, base_color: Optional[BaseColor] = None, **delivery_opts):
         '''
-        sets the color of the current grid at the given row and column
+        sets the color of the current grid at the given row and column or at the given cell_number
+
+        Optional
+        ----------
+        row : int
+            row of the cell
+
+        column : int
+            column of the cell
+
+        cell_number : int
+            cells are enumerated from left to right, starting with "1" on cell top left
+
+        color : CssColorType
+            color to set
+
+        base_color : CssColorType
+            base color when numeric color values are set
         '''
+        self.update_grid(row=row, column=column, color=color, cell_number=cell_number,
+                         base_color=base_color, **delivery_opts)
+
+    def set_grid_at(self, row: Optional[int] = None, column: Optional[int] = None, color: Optional[CssColorType] = None, cell_number: Optional[int] = None, base_color: Optional[BaseColor] = None, **delivery_opts):
+        '''
+        sets the color of the current grid at the given row and column or at the given cell_number
+
+        Optional
+        ----------
+        row : int
+            row of the cell
+
+        column : int
+            column of the cell
+
+        cell_number : int
+            cells are enumerated from left to right, starting with "1" on cell top left
+
+        color : CssColorType
+            color to set
+
+        base_color : CssColorType
+            base color when numeric color values are set
+        '''
+
+        self.update_grid(row=row, column=column, color=color, cell_number=cell_number,
+                         base_color=base_color, **delivery_opts)
+
+    def update_grid(self, row: Optional[int] = None, column: Optional[int] = None, color: Optional[CssColorType] = None, cell_number: Optional[int] = None, base_color: Optional[BaseColor] = None, enumerate: bool = None, **delivery_opts):
+        '''
+        sets the color of the current grid at the given row and column or at the given cell_number
+
+        Optional
+        ----------
+        row : int
+            row of the cell
+
+        column : int
+            column of the cell
+
+        cell_number : int
+            cells are enumerated from left to right, starting with "1" on cell top left
+
+        color : CssColorType
+            color to set
+
+        base_color : CssColorType
+            base color when numeric color values are set
+
+        enumerate : bool
+            wheter to enumerate grid cells or not
+        '''
+
         grid_msg = {
             'type': 'grid_update',
             'row': row,
             'column': column,
-            'number': cellNumber,
+            'number': cell_number,
             'color': color,
             'base_color': base_color,
             'enumerate': enumerate,
             'time_stamp': self.current_time_stamp
         }
+        self.__set_local_grid_at(row=row, column=column, color=color, cell_number=cell_number)
         self.emit(
             SocketEvents.NEW_DATA,
             without_none(grid_msg),
@@ -990,13 +1071,32 @@ class Connector:
     def reset_grid(self, **delivery_opts):
         self.set_grid([['white']], **delivery_opts)
 
-    def __set_local_grid_at(self, row: int, column: int, color: CssColorType):
+    def setup_grid(self, rows: int, columns: int, color: Optional[CssColorType] = None, enumerate: Optional[bool] = None, **delivery_opts):
+        grid = []
+        colr = color if color is not None else 0
+        for _ in range(rows):
+            row = []
+            for _ in range(columns):
+                row.append(colr)
+            grid.append(row)
+        self.set_grid(grid, enumerate=enumerate, base_color='white', **delivery_opts)
+
+    def __set_local_grid_at(self, row: Optional[int] = None, column: Optional[int] = None, color: Optional[CssColorType] = None, cell_number: Optional[int] = None):
         grid = self.get_grid
 
         rows = len(grid)
         if rows == 0:
             grid = [[]]
 
+        col_count = len(grid[0]) if grid[0] is not None else 0
+        if cell_number is not None:
+            col_cnt = col_count
+            if col_cnt == 0:
+                col_cnt = 1
+            cell_number = cell_number - 1
+            row = cell_number // col_cnt
+            column = cell_number % col_cnt
+
         while len(grid[0]) <= column:
             for col in grid:
                 col.append(0)
@@ -1314,20 +1414,20 @@ class Connector:
             return
 
         if data['device_id'] not in self.data:
-            self.data[data['device_id']] = {}
+            self.data[data['device_id']] = DictX({})
 
         if data['type'] not in self.data[data['device_id']]:
             self.data[data['device_id']][data['type']] = []
 
-        self.__update_current_data_frame(data)
-        self.__update_latest_data(data)
-
         data_len = len(self.data[data['device_id']][data['type']])
         if not self.__record_data and (data_len >= data_threshold(data['type'])):
             self.data[data['device_id']][data['type']].pop(0)
 
         self.data[data['device_id']][data['type']].append(cast(ClientMsg, data))
 
+        self.__update_current_data_frame(data)
+        self.__update_latest_data(data)
+
         if self.__main_thread_blocked:
             self.__blocked_data_msgs.append(cast(DataMsg, data))
         else:
